==> kylin-app-manager_3.21.0.1.orig.tar.gz: kylin-app-manager-3.21.0.1/kylin-app-manager/src/dbusservices/applauncher.h <==
/*
 * Copyright 2022 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef APPLAUNCHER_H
#define APPLAUNCHER_H

#include <QObject>
#include <QMap>
#include <QPair>
#include <QList>
#include <KDesktopFile>
#include <QSettings>
#include <QTimer>
#include <QFile>
#include <QtConcurrent>
#include <QFuture>
#include <QMutex>
#include <QDebug>
#include <kwindowsystem.h>
#include <KWindowInfo>
#include <KConfigGroup>
#include <QMimeDatabase>
#include <QVector>
#include "utils/processinfo.h"
#include "dbusinterfaces/cgroupinterface.h"
#include "dbusinterfaces/kwininterface.h"
#include "dbusinterfaces/statusmanagerinterface.h"
#include "dbusservices/whitelistmanager.h"
#include "applauncherdaemon.h"
#include "windowmanager/windowmanager.h"

using namespace kdk;

extern "C" {
#include <gio/gdesktopappinfo.h>
}

class QProcess;
class QTimer;
class QFileSystemWatcher;
class ChildrenProcessWatcher;
class AppLauncherDaemon;
class WhiteListManager;
class ProcessInfo;
class AppLauncher : public QObject
{
    Q_OBJECT
public:

    struct Timer {
        quint64 deadCounter;
        QString desktopFile;
        bool operator==(const Timer &l) const {
            return l.deadCounter == this->deadCounter &&
                    l.desktopFile == this->desktopFile;
        }
    };
    typedef QList<ProcessInfo> ProcessList ;
    typedef QMap<QString, ProcessList> AppsInfo;

    explicit AppLauncher(QObject *parent = nullptr);

    bool LaunchApp(const QString &desktopFile);
    bool LaunchAppWithArguments(const QString &desktopFile,const QStringList &args);
    bool Open(const QString &fileName);
    bool LaunchDefaultAppWithUrl(const QString &url);
    bool ActiveProcessByWid(const WindowId &wid);
    QString AppDesktopFileNameByPid(qint64 pid);
    QString AppDesktopFileNameByWid(qint64 wid);
    /**
     * @brief RecommendAppLists 获取可打开文件的推荐应用列表
     * @param fileName 文件名称
     * @return 返回默认打开应用和推荐打开应用的列表容器
     */
    QVector<QStringList> RecommendAppLists(const QString &fileName);

#if 0
    QList<uint> RunningApplications();
#endif
    void raiseApplication();
    bool raiseOnCurrentDesktop();

    ///
    /// \brief setPowerSavingModeEnable 打开/关闭省电模式
    /// \param enable 为 true 时打开省电模式
    ///
    void setPowerSavingModeEnable(bool enable);

public Q_SLOTS:
    ///
    /// \brief killCachedApp 结束掉处理缓存状态的app
    ///
    void killCachedApp();
    void updateWhitelistCGroupStatus(const QString &desktopName);

private Q_SLOTS:
    void tabletModeChanged(bool tabletMode);
    void activeWindowChanged(const WindowId &window);
#ifdef INVISIBLE_ENABLE
    void timeOut();
    void handleTimeOut();
#endif
    void aboutToQuitApp();
    void windowAdded(const WindowId &window);
    void windowRemoved(const WindowId &window);
    void updateDesktopExecsInfo(const QString &path);

Q_SIGNALS:
    void aboutToQuitMainApp();
    void kmreAppPid();
    void terminalPid(int pid, WindowId wid);
    void tencentAppSignal(int pid, WindowId wid);
private:
    void initDesktopFileWatcher();
    void initConnections();

    bool updateAppCGroupStatus(int pid, ProcessInfo::AppCGroup status);
    void updateAppCGroupStatus();

    /**
     * @brief desktopFileExec 获取desktop文件中Exec字段
     * @param desktopFileName desktop名称
     * @return Exec字段
     */
    QPair<QString, QStringList> desktopFileExec(const QString &desktopFileName);

    ///
    /// \brief isKmreApp 根据desktop文件判断当前应用是不是kmre的应用
    /// \param desktopFile
    /// \return
    ///
    bool isKmreApp(const QString &desktopFile);

    ///
    /// \brief hasWindow 根据desktop文件判断当前文件是否会显示窗口
    /// \param desktopFile
    /// \return
    ///
    bool hasWindow(const QString &desktopFile);

#ifdef INVISIBLE_ENABLE
    ///
    /// \brief addTimer 当 \param desktopFile 应用处于 InvisibleProcess 状态时，
    ///  添加定时器时间，超时后移动至 CachedProcess 分组
    /// \param desktopFile
    ///
    void addTimer(const QString &desktopFile);
    void deleteTimer(const QString &desktopFile);
#endif

    /**
     * @brief desktopFullFilename   获得带全路径的desktop名称
     * @param desktopFileName   应用的desktop名称
     * @return  带全路径的desktop名称
     */
    QString desktopFullFilename(const QString &desktopFileName);

    /**
     * @brief doLaunchApp   启动应用
     * @param desktopFile   应用的desktop名称
     * @param args  启动参数
     * @return  启动成功返回true
     */
    bool doLaunchApp(const QString &desktopFile, const QStringList &args = QStringList());

    /**
     * @brief launch 打开应用
     * @param execPath 应用名称
     * @param args 启动参数
     * @return 启动成功返回true
     */
    bool launch(const QString &execPath, const QStringList &args = QStringList());

    /**
     * @brief launchDesktopApp 优先使用GIO的接口打开应用，如果打开失败尝试调用 \memberof launch
     * @param desktopFile 绝对路径desktop文件
     * @param args 指定的参数
     * @param execPath 应用的启动二进制文件，用于 \memberof launch函数
     * @param execArgs 启动参数，用于 \memberof launch函数
     * @return
     */
    bool launchDesktopApp(const QString &desktopFile,
                          const QStringList &args,
                          const QString &execPath,
                          const QStringList &execArgs);

    /**
     * @brief appIdByPid 获取指定进程pid对应的应用的id，desktop文件或者app的名称
     * @param pid 根据WId获取到的pid
     * @param appStatus 当前进程处于的状态
     * @return <desktopName, index>
     */
    QPair<QString, int> appIdByPid(quint32 pid, ProcessInfo::AppStatus appStatus);

    ///
    /// \brief activeAppWindow 激活已经打开的应用的窗口
    /// \param desktopFile 应用对应的desktop文件
    /// \param args 启动参数
    /// \return 激活成功返回true
    ///
    bool activeAppWindow(const QString &desktopFile, const QStringList &args);
    bool activeAppWindow(const QString &desktopFile);
    bool activePolkitWindow();

    ProcessInfo::AppStatus appStatus(const QString &desktopFile, const QStringList &args);
    ProcessInfo::AppStatus appStatus(const QString &desktopFile);

    /**
     * @brief tryToActiveExistWindow 可能存在desktop文件不同，但是启动同一个应用的情况，比如 wps
     * @param args 启动的参数
     * @return 激活成功返回true
     */
    bool tryToActiveExistWindow(const QStringList &args);

    /**
     * @brief launchingTimeout 判断应用是否拉起超时
     *  超时分两种情况：1、应用自身的问题，没正常打开；2、多个desktop文件启动的是同一个应用
     * @param desktop desktop文件
     * @param args 对应的启动参数
     * @return 超时返回true
     */
    bool launchingTimeout(const QString &desktop, const QStringList &args);

    /**
     * @brief doActiveProcessByWid  提供给多任务视图互动关闭时将进程移动至前台分组
     * @param wid   滑动关闭应用的进程
     * @return  移动成功返回true
     */
    bool doActiveProcessByWid(const WindowId &wid);

    /**
     * @brief windowChanged 根据kf5的窗口添加或者删除的信号进行处理
     * @param wid   窗口wid
     * @param added 窗口添加为true，删除为false
     */
    void windowChanged(WindowId wid, bool added);

    /**
     * @brief moveCachePidToGroup 保护机制，防止一些僵尸进程和线程一直保存在缓存分组中
     */
    void moveCachePidToGroup(ProcessInfo::AppCGroup status);

    static void childProcessCallback(GDesktopAppInfo *appInfo, GPid pid, gpointer userData);

    /**
     * @brief getLaunchAppLists 利用gio的接口获取文件的默认打开应用和推荐打开应用列表
     * @param fileName 文件名称
     * @return 返回可打开文件的应用列表
     */
    QVector<QStringList> getLaunchAppLists(const QString &fileName);

    /**
     * @brief isWakeupHoverButtonApp 判断打开的应用是否为需要唤醒悬浮按钮类应用
     * @param fullDesktopName 应用的desktop名称
     * @return 如果包含悬浮按钮类应用则为true
     */
    bool isWakeupHoverButtonApp(const QString &fullDesktopName);

    /**
     * @brief isWakeupHoverButtonApp 根据传入的pid的cmdline信息判断是否为唤醒悬浮按钮类应用
     * @param pid 应用进程pid
     * @return 如果包含悬浮按钮类应用则为true
     */
    bool isWakeupHoverButtonApp(const int &pid);


private:
    bool m_isTabletMode;
    static AppsInfo m_appsInfo;
    bool m_isWakeupHoverButton;

#ifdef INVISIBLE_ENABLE
    QTimer *m_timer;
    // contains the time when the app would be cached(CachedProcess)
    QList<Timer> m_timerContainer;
    quint64 currentCounter;
#endif
    bool m_powerSavingEnabled;
    com::kylin::CGroup::AppManager *m_cgroupInterface;
    org::ukui::KWin *m_kwinInterface;
    com::kylin::statusmanager::interface *m_statusManagerInterface;
    AppLauncherDaemon *m_appLauncherDaemon;
    WhiteListManager *m_whiteListManager;
    QFileSystemWatcher *m_desktopFileWatcher;
    QMap<QString, QPair<QString, QStringList>> m_desktopExec;
    QMutex m_mutex;
    int m_polkitWindowPid;
    WindowId m_polkitWindowId;

    const quint64 interval = 10;//30 * 60;
    const QString kKmreAppMainWindow = "/usr/bin/kylin-kmre-window";
    const QString polkitWindowPidCmdline =
            "/usr/lib/x86_64-linux-gnu/ukui-polkit/polkit-ukui-authentication-agent-1";
    const QStringList kNeedingPolkitWindowApps = {
        "gparted.desktop"
    };
    const QStringList kFreeApps = {
        "ukui-search-menu.desktop"
    };

    struct {
        int maxname;
        int fd[2];
        int launcher[2]; /* socket pair for launcher communication */
        int deadpipe[2]; /* pipe used to detect dead children */
        int initpipe[2];
        int wrapper; /* socket for wrapper communication */
        int accepted_fd; /* socket accepted and that must be closed in the child process */
        char result;
        int exit_status;
        pid_t fork;
        pid_t launcher_pid;
        pid_t kded_pid;
        int n;
        char **argv = nullptr;
        int (*func)(int, char *[]);
        int (*launcher_func)(int);
        bool debug_wait;
        QByteArray errorMsg;
        bool launcher_ok;
        bool suicide;
    } d;
};

#endif // APPLAUNCHER_H

==> kylin-app-manager_3.22.1.0.orig.tar.gz: kylin-app-manager-3.22.1.0/kylin-app-manager/src/dbusservices/applauncher.h <==
/*
 * Copyright 2022 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef APPLAUNCHER_H
#define APPLAUNCHER_H

#include <QObject>
#include <QMap>
#include <QPair>
#include <QList>
#include <KDesktopFile>
#include <QSettings>
#include <QTimer>
#include <QFile>
#include <QtConcurrent>
#include <QFuture>
#include <QMutex>
#include <QDebug>
#include <kwindowsystem.h>
#include <KWindowInfo>
#include <KConfigGroup>
#include <QMimeDatabase>
#include <QVector>
#include <QJsonObject>
#include <QFileSystemWatcher>
#include "utils/processinfo.h"
#include "dbusinterfaces/cgroupinterface.h"
#include "dbusinterfaces/kwininterface.h"
#include "dbusservices/whitelistmanager.h"
#include "common.h"
#include "core/appinfomanager.h"

extern "C" {
#include <gio/gdesktopappinfo.h>
}

class QProcess;
class QTimer;
class QFileSystemWatcher;
class ChildrenProcessWatcher;
class AppLauncherDaemon;
class WhiteListManager;
class ProcessInfo;
class AppLauncher : public QObject
{
    Q_OBJECT
public:
    typedef QList<ProcessInfo> ProcessList ;
    typedef QMap<QString, ProcessList> AppsInfo;

    explicit AppLauncher(QObject *parent = nullptr);

    bool LaunchApp(const QString &desktopFile);
    bool LaunchAppWithArguments(const QString &desktopFile,const QStringList &args);
    bool Open(const QString &fileName);
    bool LaunchDefaultAppWithUrl(const QString &url);
    bool ActiveProcessByWid(const uint &wid);
    bool ActiveProcessByPid(const int &pid);
    QString AppDesktopFileNameByPid(qint64 pid);
    QString AppDesktopFileNameByWid(qint64 wid);
    /**
     * @brief RecommendAppLists 获取可打开文件的推荐应用列表
     * @param fileName 文件名称
     * @return 返回默认打开应用和推荐打开应用的列表容器
     */
    QVector<QStringList> RecommendAppLists(const QString &fileName);

    /**
     * @brief setEnvValue 设置环境变量的值
     * @param envValue 环境变量value
     */
    void setEnvValue(const QByteArray &envValue);

    /**
     * @brief isDesktopFileNoDisplay 判断desktop文件中的NoDisplay字段
     * @param desktopFileName desktop名称
     * @return  字段为true返回true
     */
    bool isDesktopFileNoDisplay(const QString &desktopFileName);

public Q_SLOTS:
    void wakeupHoverButton(const QString &name, const QString &binary, const QStringList &args);

Q_SIGNALS:
    void aboutToQuitMainApp();

private:
    /**
     * @brief desktopFileExec 获取desktop文件中Exec字段
     * @param desktopFileName desktop名称
     * @return Exec字段
     */
    QPair<QString, QStringList> desktopFileExec(const QString &desktopFileName);

    ///
    /// \brief hasWindow 根据desktop文件判断当前文件是否会显示窗口
    /// \param desktopFile
    /// \return
    ///
    bool hasWindow(const QString &desktopFile);

    /**
     * @brief desktopFullFilename   获得带全路径的desktop名称
     * @param desktopFileName   应用的desktop名称
     * @return  带全路径的desktop名称
     */
    static QString desktopFullFilename(const QString &desktopFileName);

    /**
     * @brief doLaunchApp   启动应用
     * @param desktopFile   应用的desktop名称
     * @param args  启动参数
     * @return  启动成功返回true
     */
    bool doLaunchApp(const QString &desktopFile, const QStringList &args = QStringList());

    /**
     * @brief launch 打开应用
     * @param execPath 应用名称
     * @param args 启动参数
     * @return 启动成功返回true
     */
    bool launch(const QString &desktopFile,
                const QString &execPath,
                const QStringList &args = QStringList());

    /**
     * @brief launchDesktopApp 优先使用GIO的接口打开应用，如果打开失败尝试调用 \memberof launch
     * @param desktopFile 绝对路径desktop文件
     * @param args 指定的参数
     * @param execPath 应用的启动二进制文件，用于 \memberof launch函数
     * @param execArgs 启动参数，用于 \memberof launch函数
     * @return 启动成功返回true
     */
    bool launchDesktopApp(const QString &desktopFile,
                          const QStringList &args,
                          const QString &execPath,
                          const QStringList &execArgs);

    /**
     * @brief activeAppWindow
     * @param wid
     * @param pids
     * @return
     */
    bool activeAppWindow(quint32 wid, QList<int> pids);
    bool activePolkitWindow();

    static void childProcessCallback(GDesktopAppInfo *appInfo, GPid pid, gpointer userData);

    /**
     * @brief getLaunchAppLists 利用gio的接口获取文件的默认打开应用和推荐打开应用列表
     * @param fileName 文件名称
     * @return 返回可打开文件的应用列表
     */
    QVector<QStringList> getLaunchAppLists(const QString &fileName);

private:
    QByteArray m_envValue;
    org::ukui::KWin *m_kwinInterface;
    bool m_isWayland;
//    QMap<QString, QPair<QString, QStringList>> m_desktopExec;
    QMutex m_mutex;
    int m_polkitWindowPid;
    uint m_polkitWindowId;
    AppInfoManager &m_appInfoManager = common::Singleton<AppInfoManager>::GetInstance();

    const quint64 interval = 10;//30 * 60;
    const QString kKmreAppMainWindow = "/usr/bin/kylin-kmre-window";
    const QStringList polkitWindowPidCmdline = {
        "/usr/lib/x86_64-linux-gnu/ukui-polkit/polkit-ukui-authentication-agent-1",
        "/usr/lib/aarch64-linux-gnu/ukui-polkit/polkit-ukui-authentication-agent-1"
    };
    const QStringList kNeedingPolkitWindowApps = {
        "gparted.desktop"
    };
    const QStringList kFreeApps = {
        "ukui-search-menu.desktop",
        "onboard.desktop"
    };
    struct {
        int maxname;
        int fd[2];
        int launcher[2]; /* socket pair for launcher communication */
        int deadpipe[2]; /* pipe used to detect dead children */
        int initpipe[2];
        int wrapper; /* socket for wrapper communication */
        int accepted_fd; /* socket accepted and that must be closed in the child process */
        char result;
        int exit_status;
        pid_t fork;
        pid_t launcher_pid;
        pid_t kded_pid;
        int n;
        char **argv = nullptr;
        int (*func)(int, char *[]);
        int (*launcher_func)(int);
        bool debug_wait;
        QByteArray errorMsg;
        bool launcher_ok;
        bool suicide;
    } d;
};

#endif // APPLAUNCHER_H
